@@ -35,52 +35,6 @@ sub apply_node_navopts {}
 has '+fetch_nodes_deep', default => 0;
 
 
-has '+node_types', default => sub {[
-  {
-    type     => 'folder',
-    title    => 'Folder',
-    iconCls  => 'ra-icon-folder',
-    addable  => 1,
-    editable => 1
-  },
-  {
-    type     => 'search',
-    title    => 'Saved Search',
-    addable  => 0,
-    editable => 1
-  },
-  {
-    type     => 'link',
-    title    => 'Custom Link',
-    iconCls  => 'ra-icon-link-go',
-    addable  => 1,
-    editable => 1,
-    applyDialogOpts => {
-      height => 200,
-    },
-    fields => [{
-      name  => 'iconcls',
-      xtype => 'ra-all-icon-assets-combo',
-      value => 'ra-icon-link-go',
-      fieldLabel => 'Icon',
-    },{
-      name  => 'url', 
-      xtype => 'textfield',
-      fieldLabel => 'Link URL',
-      plugins => [{
-        ptype => 'fieldhelp',
-        text  => "Local internal URL path, starting with '/'"
-      }],
-      allowBlank => \0,
-      validator => jsfunc join(' ','function(v) {',
-        'return (v && v.search("/") == 0) ? true : false;',
-      '}')
-    }]
-  },
-
-]};
-
-
 sub auth_active {
   my $self = shift;
   return $self->c->can('user') ? 1 : 0;
@@ -103,10 +57,19 @@ sub BUILD {
 	$self->root_node->{allowAdd} = \0;
 	
 	$self->add_ONCONTENT_calls('apply_permissions');
+  
+  $self->add_plugin('ra-navcore-import-defaults');
+  $self->apply_actions( import_defaults => 'import_defaults_action' );
 }
 
 sub apply_permissions {
 	my $self = shift;
+  
+  # This is what causes the 'ra-navcore-import-defaults' plugin to become active:
+  $self->apply_extconfig( 
+    import_defaults_url => $self->suburl('import_defaults') 
+  ) if ($self->is_import_allowed);
+  
 	return if ($self->can_edit_navtree);
 	
 	$self->apply_extconfig(
@@ -115,6 +78,28 @@ sub apply_permissions {
 	);
 }
 
+has 'default_public_nav_items', is => 'ro', lazy => 1, default => sub {
+  my $self = shift;
+  my $cfg  = clone($self->app->config->{'Plugin::RapidApp::NavCore'} || {});
+  $cfg->{default_public_nav_items}
+};
+
+sub is_import_allowed {
+  my $self = shift;
+  $self->is_admin 
+    and $self->default_public_nav_items 
+    and $self->c->can('_navcore_init_default_public_nav_items')
+}
+
+sub import_defaults_action {
+  my $self = shift;
+  die usererr "PERMISSION DENIED" unless ($self->is_import_allowed);
+  die "Malformed request" unless $self->c->req->params->{navcore_confirm_import};
+  
+  $self->c->_navcore_init_default_public_nav_items(1)
+}
+
+
 
 sub fetch_nodes {
 	my $self = shift;
@@ -311,7 +296,8 @@ sub add_node {
     $Node = $self->Rs->create({
       pid => $id,
       text => $name,
-      ordering => $order
+      ordering => $order,
+      iconcls => $params->{iconcls}
     });
   }
 	
@@ -641,10 +627,16 @@ sub rename_node {
 	#my $Node = $self->Rs->search_rs({ 'me.id' => $id })->first;
 	$Node->update({ 'text' => $name });
 	
+  $Node->text($name);
+  $Node->iconcls($params->{iconcls}) if ($params->{iconcls});
+  
+  $Node->update;
+  
 	return {
 		msg		=> 'Renamed',
 		success	=> \1,
 		new_text => $Node->text,
+    new_iconcls => $Node->iconcls
 	};
 }
 
@@ -655,10 +647,11 @@ sub rename_search {
   
   my $update = { title => $name };
   if($nodeTypeName eq 'link') {
-    $update->{url}     = $params->{url}     if ($params->{url});
-    $update->{iconcls} = $params->{iconcls} if ($params->{iconcls});
+    $update->{url} = $params->{url} if ($params->{url});
   }
   
+  $update->{iconcls} = $params->{iconcls} if ($params->{iconcls});
+  
   if ($State->update($update)) {
     my $res = {
       msg		=> 'Renamed Search',
